Whether it's ticking a checkbox, selecting a dropdown, or entering text into the sidebar or options (or popup), and you're saving that to Storage API, sometimes you want the update to apply immediately (if another presentation renders your Storage data).

---

## Reorientation

Let's continue the code from [[1b. Save options that affect the rest of Chrome experience]]

If you have not, install the Chrome Extension from that challenge.

Go open Options and set all webpages to the background color blue.

Visiting google.com shows a blue background (Recall that some websites have all layouts with their own colors and therefore the body background doesn't show through):
![[Pasted image 20250405031456.png]]

Now open Options and change the background color to red. Open the tab to our google.com. Notice that the background is still blue:
![[Pasted image 20250405031456.png]]


You have to hit REFRESH for the new background color to load red. 
![[Pasted image 20250406002323.png]]

What if this were instant the moment the storage value is changed? That's what Storage's onChange is for

---

## Storage onChange

We can have a background.js that listens for storage onChange, and then run applyBackgroundColor() using background's `chrome.scripting.executeScript()`. That will effectively run the applyBackgroundColor() that's in content.js because that's a global script attached to the web content.

The problem is that Options is on a separate tab from your web content and background needs to be running at the tab where the web content needing the changes are at. This means the Options have to be moved to a popup, effectively making the dropdown for background color and the webpage needing to have real time changes - to be on the same tab:
![[Pasted image 20250406003318.png]]

Because the popup is still opened the moment you finished selecting the new background color, then background.js actively runs the code needed to update the active tab's web content background color. Background.js runs as long as we have a Chrome Extension presentation opened.

At manifest, we removed options, and then we added scripting permissions, an action popup, and the background.js. Your manifest.json could look like:
```
{  
    "manifest_version": 3,  
    "name": "<APP_NAME>",  
    "version": "1.0",  
    "description": "<APP_DESCRIPTION>.",  
    "author": "Weng Fei Fung",
    "icons": {  
        "16": "icon16x16.png",  
        "32": "icon32x32.png",  
        "48": "icon48x48.png",  
        "128": "icon128x128.png"  
    },
    "content_security_policy": {  
        "extension_pages": "default-src 'self'; script-src 'self'; style-src 'self'; img-src 'self'; object-src 'self';"
    },
    "host_permissions": [
        "https://*/*",
        "http://*/*"
    ],
    "content_scripts": [
        {
	        "matches": ["https://*/*", "http://*/*"],
	        "js": ["content.js"],
	        "run_at": "document_end"
        }
    ],
    "permissions": [
        "storage", "tabs", "activeTab", "scripting"
    ],
    "action": {
        "default_popup": "options.html"
    },
    "background": {
        "service_worker": "background.js"
    }
}
```

^ Note if you hadn't added scripting, the background.js' `chrome.scripting.executeScript` will complain with this vague error:
```
Error handling response: TypeError: Cannot read properties of undefined (reading 'executeScript')  
    at chrome-extension://mecibbiemapapihbpgkginjgbieifhhh/background.js:8:30
```

We leave options.html alone since that'll be the popup path. We also leave options.js and options.css alone because they're loaded by options.html for logic and presentation purposes.

Create background.js:
```
chrome.storage.onChanged.addListener(function (changes, areaName) {
    if (areaName === 'local' && changes.backgroundColor) {
        const { oldValue, newValue } = changes.backgroundColor;
        console.log(`Background detected Storage change for backgroundColor from ${oldValue} to ${newValue}`);

        chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
        if (tabs[0]) {
            const tabId = tabs[0].id;
            chrome.scripting.executeScript({
                target: {tabId: tabId},
                func: () => {
                    applyBackgroundColor();
                }
            });
        }
        });
    } // if areaName is local and changes.backgroundColor
  });
```

^ FYI, the areaName could be local, sync, or managed. We are local only, meaning Storage API does not share the data with your other devices whose Chrome is signed in.

---

## Testing

Update your chrome extension.

At google.com, open the popup by clicking the Chrome Extension icon. Try changing the background color. The changes at google.com should be happening in real time. No need to refresh the page to see the changes.

![[Chrome-extension-storage-onchange.gif]]